"""
门墩儿数据解析模块

该模块提供门墩儿人才数据的解析和处理功能。
"""

import logging
from datetime import datetime
import json
import os
from typing import Dict, Any, Optional, List
import re


def parse_menduner_data(data_source: str, data_type: str = 'json') -> Dict[str, Any]:
    """
    解析门墩儿人才数据
    
    Args:
        data_source (str): 数据源（文件路径或JSON字符串）
        data_type (str): 数据类型，可选值：'json', 'file', 'api'
        
    Returns:
        Dict[str, Any]: 解析结果
    """
    try:
        logging.info(f"开始解析门墩儿数据，类型: {data_type}")
        
        raw_data = None
        
        if data_type == 'file':
            # 从文件读取数据
            if not os.path.exists(data_source):
                return {
                    'success': False,
                    'error': f'数据文件不存在: {data_source}',
                    'data': None
                }
            
            with open(data_source, 'r', encoding='utf-8') as f:
                raw_data = f.read()
                
        elif data_type == 'json':
            # 直接处理JSON字符串
            raw_data = data_source
            
        elif data_type == 'api':
            # TODO: 实现API数据获取逻辑
            raw_data = data_source
        
        # 解析数据
        if raw_data:
            parsed_data = _process_menduner_content(raw_data)
            
            result = {
                'success': True,
                'error': None,
                'data': {
                    'talent_profiles': parsed_data,
                    'parse_time': datetime.now().isoformat(),
                    'source_type': data_type,
                    'total_count': len(parsed_data) if isinstance(parsed_data, list) else 1
                }
            }
            
            logging.info(f"门墩儿数据解析完成，共解析 {result['data']['total_count']} 条记录")
            return result
        
        return {
            'success': False,
            'error': '无法获取有效数据',
            'data': None
        }
        
    except Exception as e:
        error_msg = f"解析门墩儿数据失败: {str(e)}"
        logging.error(error_msg, exc_info=True)
        
        return {
            'success': False,
            'error': error_msg,
            'data': None
        }


def _process_menduner_content(raw_content: str) -> List[Dict[str, Any]]:
    """
    处理门墩儿原始数据内容
    
    Args:
        raw_content (str): 原始数据内容
        
    Returns:
        List[Dict[str, Any]]: 处理后的人才档案列表
    """
    try:
        # 尝试解析JSON格式
        try:
            json_data = json.loads(raw_content)
            if isinstance(json_data, list):
                return [_normalize_talent_profile(item) for item in json_data]
            elif isinstance(json_data, dict):
                return [_normalize_talent_profile(json_data)]
        except json.JSONDecodeError:
            pass
        
        # 如果不是JSON，尝试按行解析
        lines = raw_content.strip().split('\n')
        profiles = []
        
        for line in lines:
            line = line.strip()
            if line:
                profile = _parse_talent_line(line)
                if profile:
                    profiles.append(profile)
        
        return profiles
        
    except Exception as e:
        logging.error(f"处理门墩儿数据内容失败: {str(e)}")
        return []


def _normalize_talent_profile(raw_profile: Dict[str, Any]) -> Dict[str, Any]:
    """
    标准化人才档案数据
    
    Args:
        raw_profile (Dict[str, Any]): 原始档案数据
        
    Returns:
        Dict[str, Any]: 标准化后的档案数据
    """
    normalized = {
        'name': raw_profile.get('name', ''),
        'phone': _normalize_phone(raw_profile.get('phone', '')),
        'email': _normalize_email(raw_profile.get('email', '')),
        'position': raw_profile.get('position', ''),
        'company': raw_profile.get('company', ''),
        'location': raw_profile.get('location', ''),
        'experience_years': raw_profile.get('experience_years', 0),
        'skills': raw_profile.get('skills', []),
        'education': raw_profile.get('education', ''),
        'source': 'menduner',
        'processed_time': datetime.now().isoformat(),
        'raw_data': raw_profile
    }
    
    return normalized


def _parse_talent_line(line: str) -> Optional[Dict[str, Any]]:
    """
    解析单行人才信息
    
    Args:
        line (str): 包含人才信息的文本行
        
    Returns:
        Optional[Dict[str, Any]]: 解析后的人才信息，如果解析失败返回None
    """
    try:
        # TODO: 根据门墩儿数据的具体格式实现解析逻辑
        # 这里提供一个基础的解析示例
        
        # 简单的分隔符解析（假设用制表符或逗号分隔）
        if '\t' in line:
            parts = line.split('\t')
        elif ',' in line:
            parts = line.split(',')
        else:
            parts = [line]
        
        if len(parts) >= 3:
            return {
                'name': parts[0].strip(),
                'phone': parts[1].strip() if len(parts) > 1 else '',
                'position': parts[2].strip() if len(parts) > 2 else '',
                'company': parts[3].strip() if len(parts) > 3 else ''
            }
        
        return None
        
    except Exception as e:
        logging.error(f"解析人才信息行失败: {str(e)}")
        return None


def _normalize_phone(phone: str) -> str:
    """
    标准化电话号码格式
    
    Args:
        phone (str): 原始电话号码
        
    Returns:
        str: 标准化后的电话号码
    """
    if not phone:
        return ''
    
    # 移除所有非数字字符
    digits = re.sub(r'\D', '', phone)
    
    # 中国手机号码格式化
    if len(digits) == 11 and digits.startswith('1'):
        return f"{digits[:3]}-{digits[3:7]}-{digits[7:]}"
    
    return phone


def _normalize_email(email: str) -> str:
    """
    标准化邮箱地址
    
    Args:
        email (str): 原始邮箱地址
        
    Returns:
        str: 标准化后的邮箱地址
    """
    if not email:
        return ''
    
    # 基础邮箱格式验证
    email_pattern = r'^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$'
    if re.match(email_pattern, email.strip()):
        return email.strip().lower()
    
    return email


def validate_menduner_data(data: Dict[str, Any]) -> Dict[str, Any]:
    """
    验证门墩儿人才数据的完整性和有效性
    
    Args:
        data (Dict[str, Any]): 待验证的人才数据
        
    Returns:
        Dict[str, Any]: 验证结果
    """
    try:
        errors = []
        warnings = []
        
        # 必填字段检查
        required_fields = ['name']
        for field in required_fields:
            if not data.get(field):
                errors.append(f"缺少必填字段: {field}")
        
        # 可选但建议填写的字段
        recommended_fields = ['phone', 'position', 'company']
        for field in recommended_fields:
            if not data.get(field):
                warnings.append(f"建议填写字段: {field}")
        
        # 格式验证
        if data.get('phone'):
            phone = data['phone']
            if not re.match(r'^1[3-9]\d{9}$', re.sub(r'\D', '', phone)):
                warnings.append("电话号码格式可能不正确")
        
        if data.get('email'):
            email = data['email']
            if not re.match(r'^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$', email):
                errors.append("邮箱格式不正确")
        
        return {
            'is_valid': len(errors) == 0,
            'errors': errors,
            'warnings': warnings,
            'score': max(0, 100 - len(errors) * 20 - len(warnings) * 5)
        }
        
    except Exception as e:
        logging.error(f"验证门墩儿数据失败: {str(e)}")
        return {
            'is_valid': False,
            'errors': [f"验证过程出错: {str(e)}"],
            'warnings': [],
            'score': 0
        }


def batch_process_menduner_data(data_list: List[Dict[str, Any]]) -> Dict[str, Any]:
    """
    批量处理门墩儿人才数据
    
    Args:
        data_list (List[Dict[str, Any]]): 待处理的人才数据列表
        
    Returns:
        Dict[str, Any]: 批量处理结果
    """
    try:
        processed_data = []
        validation_results = []
        success_count = 0
        error_count = 0
        
        for i, data in enumerate(data_list):
            try:
                # 标准化数据
                normalized = _normalize_talent_profile(data)
                
                # 验证数据
                validation = validate_menduner_data(normalized)
                
                processed_data.append(normalized)
                validation_results.append({
                    'index': i,
                    'validation': validation
                })
                
                if validation['is_valid']:
                    success_count += 1
                else:
                    error_count += 1
                    
            except Exception as e:
                error_count += 1
                validation_results.append({
                    'index': i,
                    'validation': {
                        'is_valid': False,
                        'errors': [f"处理失败: {str(e)}"],
                        'warnings': [],
                        'score': 0
                    }
                })
        
        return {
            'success': True,
            'summary': {
                'total_count': len(data_list),
                'success_count': success_count,
                'error_count': error_count,
                'success_rate': (success_count / len(data_list)) * 100 if data_list else 0
            },
            'processed_data': processed_data,
            'validation_results': validation_results
        }
        
    except Exception as e:
        error_msg = f"批量处理门墩儿数据失败: {str(e)}"
        logging.error(error_msg, exc_info=True)
        
        return {
            'success': False,
            'error': error_msg,
            'summary': None,
            'processed_data': [],
            'validation_results': []
        } 