"""
图片解析模块

该模块提供图片文件的解析功能，包括名片识别、证件照处理等。
"""

import logging
from datetime import datetime
import json
import os
from typing import Dict, Any, Optional, List, Tuple
import base64
from PIL import Image
import io


def parse_business_card_image(image_path: str, task_id: Optional[str] = None) -> Dict[str, Any]:
    """
    解析名片图片
    
    Args:
        image_path (str): 名片图片路径
        task_id (str, optional): 关联的任务ID
        
    Returns:
        Dict[str, Any]: 解析结果
    """
    try:
        logging.info(f"开始解析名片图片: {image_path}")
        
        # 验证文件存在性和格式
        validation_result = validate_image_file(image_path)
        if not validation_result['is_valid']:
            return {
                'success': False,
                'error': validation_result['error'],
                'data': None
            }
        
        # 获取图片信息
        image_info = get_image_info(image_path)
        
        # TODO: 集成OCR引擎进行名片文字识别
        # 这里应该调用OCR服务（如百度OCR、腾讯OCR等）来识别名片上的文字
        
        # 模拟名片识别结果
        card_data = _extract_business_card_info(image_path)
        
        result = {
            'success': True,
            'error': None,
            'data': {
                'personal_info': card_data,
                'image_info': image_info,
                'parse_time': datetime.now().isoformat(),
                'task_id': task_id,
                'confidence_score': 0.85  # 模拟置信度分数
            }
        }
        
        logging.info(f"名片图片解析完成: {image_path}")
        return result
        
    except Exception as e:
        error_msg = f"解析名片图片失败: {str(e)}"
        logging.error(error_msg, exc_info=True)
        
        return {
            'success': False,
            'error': error_msg,
            'data': None
        }


def parse_portrait_image(image_path: str, task_id: Optional[str] = None) -> Dict[str, Any]:
    """
    解析证件照图片
    
    Args:
        image_path (str): 证件照图片路径
        task_id (str, optional): 关联的任务ID
        
    Returns:
        Dict[str, Any]: 解析结果
    """
    try:
        logging.info(f"开始解析证件照图片: {image_path}")
        
        # 验证文件
        validation_result = validate_image_file(image_path)
        if not validation_result['is_valid']:
            return {
                'success': False,
                'error': validation_result['error'],
                'data': None
            }
        
        # 获取图片信息
        image_info = get_image_info(image_path)
        
        # 检查是否为合格的证件照
        portrait_analysis = _analyze_portrait_quality(image_path)
        
        result = {
            'success': True,
            'error': None,
            'data': {
                'image_info': image_info,
                'portrait_analysis': portrait_analysis,
                'parse_time': datetime.now().isoformat(),
                'task_id': task_id
            }
        }
        
        logging.info(f"证件照图片解析完成: {image_path}")
        return result
        
    except Exception as e:
        error_msg = f"解析证件照图片失败: {str(e)}"
        logging.error(error_msg, exc_info=True)
        
        return {
            'success': False,
            'error': error_msg,
            'data': None
        }


def validate_image_file(image_path: str) -> Dict[str, Any]:
    """
    验证图片文件的有效性
    
    Args:
        image_path (str): 图片文件路径
        
    Returns:
        Dict[str, Any]: 验证结果
    """
    try:
        # 检查文件是否存在
        if not os.path.exists(image_path):
            return {
                'is_valid': False,
                'error': f'图片文件不存在: {image_path}'
            }
        
        # 检查文件扩展名
        allowed_extensions = {'.jpg', '.jpeg', '.png', '.bmp', '.gif'}
        file_ext = os.path.splitext(image_path)[1].lower()
        
        if file_ext not in allowed_extensions:
            return {
                'is_valid': False,
                'error': f'不支持的图片格式: {file_ext}，支持的格式: {", ".join(allowed_extensions)}'
            }
        
        # 尝试打开图片验证完整性
        try:
            with Image.open(image_path) as img:
                img.verify()
        except Exception as e:
            return {
                'is_valid': False,
                'error': f'图片文件损坏或格式错误: {str(e)}'
            }
        
        return {
            'is_valid': True,
            'error': None
        }
        
    except Exception as e:
        return {
            'is_valid': False,
            'error': f'验证图片文件时出错: {str(e)}'
        }


def get_image_info(image_path: str) -> Dict[str, Any]:
    """
    获取图片基础信息
    
    Args:
        image_path (str): 图片文件路径
        
    Returns:
        Dict[str, Any]: 图片信息
    """
    try:
        with Image.open(image_path) as img:
            file_size = os.path.getsize(image_path)
            
            return {
                'filename': os.path.basename(image_path),
                'file_path': image_path,
                'file_size': file_size,
                'file_size_mb': round(file_size / (1024 * 1024), 2),
                'dimensions': {
                    'width': img.width,
                    'height': img.height
                },
                'format': img.format,
                'mode': img.mode,
                'has_transparency': img.mode in ('RGBA', 'LA') or 'transparency' in img.info
            }
            
    except Exception as e:
        logging.error(f"获取图片信息失败: {str(e)}")
        return {
            'filename': os.path.basename(image_path),
            'file_path': image_path,
            'error': str(e)
        }


def _extract_business_card_info(image_path: str) -> Dict[str, Any]:
    """
    从名片图片中提取信息（模拟实现）
    
    Args:
        image_path (str): 名片图片路径
        
    Returns:
        Dict[str, Any]: 提取的名片信息
    """
    # TODO: 这里应该集成真实的OCR服务来识别名片信息
    # 目前返回模拟数据
    
    return {
        'name': '',  # 姓名
        'title': '',  # 职位
        'company': '',  # 公司
        'department': '',  # 部门
        'phone': '',  # 电话
        'mobile': '',  # 手机
        'email': '',  # 邮箱
        'address': '',  # 地址
        'website': '',  # 网站
        'fax': '',  # 传真
        'extracted_text': '',  # 原始识别文本
        'confidence_details': {
            'name': 0.0,
            'phone': 0.0,
            'email': 0.0,
            'company': 0.0
        }
    }


def _analyze_portrait_quality(image_path: str) -> Dict[str, Any]:
    """
    分析证件照质量（模拟实现）
    
    Args:
        image_path (str): 证件照图片路径
        
    Returns:
        Dict[str, Any]: 质量分析结果
    """
    try:
        with Image.open(image_path) as img:
            width, height = img.size
            
            # 基础质量检查
            quality_checks = {
                'resolution_check': {
                    'passed': width >= 300 and height >= 400,
                    'message': f'分辨率 {width}x{height}，建议至少300x400像素'
                },
                'aspect_ratio_check': {
                    'passed': 0.7 <= (height / width) <= 1.5,
                    'message': f'宽高比 {round(height/width, 2)}，建议在0.7-1.5之间'
                },
                'format_check': {
                    'passed': img.format in ['JPEG', 'PNG'],
                    'message': f'格式 {img.format}，建议使用JPEG或PNG格式'
                }
            }
            
            # 计算总体质量分数
            passed_checks = sum(1 for check in quality_checks.values() if check['passed'])
            quality_score = (passed_checks / len(quality_checks)) * 100
            
            return {
                'quality_score': quality_score,
                'quality_level': 'excellent' if quality_score >= 90 else 
                               'good' if quality_score >= 70 else 
                               'fair' if quality_score >= 50 else 'poor',
                'checks': quality_checks,
                'recommendations': _get_portrait_recommendations(quality_checks)
            }
            
    except Exception as e:
        logging.error(f"分析证件照质量失败: {str(e)}")
        return {
            'quality_score': 0,
            'quality_level': 'unknown',
            'error': str(e)
        }


def _get_portrait_recommendations(quality_checks: Dict[str, Dict]) -> List[str]:
    """
    根据质量检查结果生成改进建议
    
    Args:
        quality_checks (Dict[str, Dict]): 质量检查结果
        
    Returns:
        List[str]: 改进建议列表
    """
    recommendations = []
    
    if not quality_checks['resolution_check']['passed']:
        recommendations.append('建议使用更高分辨率的图片，至少300x400像素')
    
    if not quality_checks['aspect_ratio_check']['passed']:
        recommendations.append('建议调整图片宽高比，使其更符合证件照标准')
    
    if not quality_checks['format_check']['passed']:
        recommendations.append('建议使用JPEG或PNG格式')
    
    if not recommendations:
        recommendations.append('图片质量良好，符合基本要求')
    
    return recommendations


def convert_image_to_base64(image_path: str) -> Optional[str]:
    """
    将图片转换为Base64编码
    
    Args:
        image_path (str): 图片文件路径
        
    Returns:
        Optional[str]: Base64编码字符串，失败时返回None
    """
    try:
        with open(image_path, 'rb') as image_file:
            encoded_string = base64.b64encode(image_file.read()).decode('utf-8')
            return encoded_string
            
    except Exception as e:
        logging.error(f"转换图片到Base64失败: {str(e)}")
        return None


def resize_image(image_path: str, max_width: int = 800, max_height: int = 600, 
                output_path: Optional[str] = None) -> Dict[str, Any]:
    """
    调整图片尺寸
    
    Args:
        image_path (str): 原始图片路径
        max_width (int): 最大宽度
        max_height (int): 最大高度
        output_path (str, optional): 输出路径，如果不指定则覆盖原文件
        
    Returns:
        Dict[str, Any]: 处理结果
    """
    try:
        with Image.open(image_path) as img:
            # 计算新尺寸，保持宽高比
            width, height = img.size
            ratio = min(max_width / width, max_height / height)
            
            if ratio < 1:  # 只有当图片超过最大尺寸时才调整
                new_width = int(width * ratio)
                new_height = int(height * ratio)
                
                resized_img = img.resize((new_width, new_height), Image.Resampling.LANCZOS)
                
                save_path = output_path or image_path
                resized_img.save(save_path, quality=95)
                
                return {
                    'success': True,
                    'original_size': (width, height),
                    'new_size': (new_width, new_height),
                    'compression_ratio': ratio,
                    'output_path': save_path
                }
            else:
                return {
                    'success': True,
                    'message': '图片尺寸已符合要求，无需调整',
                    'original_size': (width, height),
                    'output_path': image_path
                }
                
    except Exception as e:
        error_msg = f"调整图片尺寸失败: {str(e)}"
        logging.error(error_msg)
        
        return {
            'success': False,
            'error': error_msg
        }


def batch_process_images(image_paths: List[str], process_type: str = 'business_card') -> Dict[str, Any]:
    """
    批量处理图片
    
    Args:
        image_paths (List[str]): 图片路径列表
        process_type (str): 处理类型，可选值：'business_card', 'portrait'
        
    Returns:
        Dict[str, Any]: 批量处理结果
    """
    try:
        results = []
        success_count = 0
        failed_count = 0
        
        for image_path in image_paths:
            try:
                if process_type == 'business_card':
                    result = parse_business_card_image(image_path)
                elif process_type == 'portrait':
                    result = parse_portrait_image(image_path)
                else:
                    result = {
                        'success': False,
                        'error': f'不支持的处理类型: {process_type}'
                    }
                
                results.append({
                    'image_path': image_path,
                    'result': result
                })
                
                if result['success']:
                    success_count += 1
                else:
                    failed_count += 1
                    
            except Exception as e:
                failed_count += 1
                results.append({
                    'image_path': image_path,
                    'result': {
                        'success': False,
                        'error': str(e)
                    }
                })
        
        return {
            'success': True,
            'summary': {
                'total_images': len(image_paths),
                'success_count': success_count,
                'failed_count': failed_count,
                'success_rate': (success_count / len(image_paths)) * 100 if image_paths else 0
            },
            'results': results
        }
        
    except Exception as e:
        error_msg = f"批量处理图片失败: {str(e)}"
        logging.error(error_msg, exc_info=True)
        
        return {
            'success': False,
            'error': error_msg,
            'results': []
        } 